// Main

#include "tchar.h"

#include <iostream>
#include <vector>
#include <list>
#include <algorithm>
#include <conio.h>
#include <fstream>
#include <stdlib.h>
#include <time.h>
#include <chrono>
#include <sstream>
#include <thread>

#include "Graph.h"
#include "BaseGenerator.h"
#include "Partition.h"
#include "Clustering\RandomGreedy.h"
#include "Clustering\EnsembleStrategy.h"
#include "Clustering\AdaptiveRandomGreedy.h"
#include "Clustering\OptimalEnsembleStrategy.h"
#include "Clustering\AdaptiveEnsembleStrategy.h"

using namespace std;
using namespace chrono;

// Count median modularity and mean time for Graph *graph and Clutering Algorithm *alg
void test_graph(Graph* graph, ClusteringInterface *alg, int runs, double &q, double &time)
{
	vector<double> qs;
	double q1;

	high_resolution_clock::time_point t1 = high_resolution_clock::now();
	for (int i = 0; i < runs; i++)
	{
		delete alg->DetectCommunities(graph, nullptr, q1);

		qs.push_back(q1);
	}

	time = duration_cast<milliseconds>(
		high_resolution_clock::now() - t1).count() / (double)runs;
	sort(qs.begin(), qs.end());

	q = runs % 2 == 0 ? .5 * (qs[runs / 2] + qs[runs / 2 + 1]) :
		qs[runs / 2];
}


int _tmain(int argc, _TCHAR* argv[])
{
	srand(time(NULL));


	// Usage:
	Graph *graph = new Graph("1.net"); // Graph in .net or .graph format
	ClusteringInterface *alg = new RandomGreedy(10); // There are different Clustering Algorithms
	// all with their own parameters. All clustering algorithms inherit abstract class ClusteringInterface

	double t, q;

	test_graph(graph, alg, 1000, q, t);
	cout << "Modularity: " << q << endl;
	cout << "Time: " << t << endl;

	delete graph;
	delete alg;

	cout << "\nSuccess\a";
	_getch();
	return 0;
}